/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project.h"

string get_attrib(string attribute, string &text);

// Load the project file that is received by the function. If the file does
// not exist, false is returned and a message will be printed to the user.
// If it exists, each attribute is loaded.
bool ProjectSettings::load_project_file (string filename) 
{ 
  string txt, text;

  if (!g_file_test(filename.c_str(), G_FILE_TEST_EXISTS))
    return false;
    
  // Open project file
  text = read_file(filename);
  
  filename.erase(filename.rfind("/", filename.length())+1, 
                 filename.length()-filename.rfind("/", filename.length()-1));

  // Get all of the attributes
  name = get_attrib("<name>", text);
  author = get_attrib("<author>", text);
  location = get_attrib("<location>", text);
  email = get_attrib("<email>", text);
  url = get_attrib("<url>", text);
  version = get_attrib("<version>", text);
  lastfile = get_attrib("<lastfile>", text);
  sources = get_attrib("<sources>", text);
  headers = get_attrib("<headers>", text);
  other_files = get_attrib("<otherfiles>", text);
  template_name = get_attrib("<template>", text);
  
  // Create the template from the language for backward compatibility
  if (template_name == " ")
  {
    txt = get_attrib("<language>", text);
    if (txt == "+") template_name = "console-cxx";
    else template_name = "console-c";
  }
  
  // Change location to current project file location if it has changed
  if (filename != location)
  {
    location = filename;
    write_project_file();
  }

  return true; 
}

// Write the project settings to the openldev project file.
void ProjectSettings::write_project_file () 
{
  string odp, fn = location;
    
  // Create .ODP file text
  odp = "<name>" + name 
      + "*\n<author>" + author 
      + "*\n<location>" + location 
      + "*\n<email>"  + email 
      + "*\n<url>" + url 
      + "*\n<version>" + version 
      + "*\n<template>" + template_name   
      + "*\n<lastfile>" + lastfile 
      + "*\n<sources>" + sources 
      + "*\n<headers>" + headers 
      + "*\n<otherfiles>" + other_files + "*\n";
  
  fn += name + ".openldev";
  
  // Write the file
  write_file(fn, odp);
}

// Get specific attribute from the text. The function receives the attribute
// and the text of the project file. It returns that text of the attribute.
string get_attrib (string attribute, 
                   string &text)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    return text.substr(start, len);
  }
  
  return " "; 
}

// Close the project by clearing all of the files, but first write the
// project file
void ProjectSettings::close_project()
{
  write_project_file();
  set_source_files("");
  set_header_files("");
  set_other_files("");
  set_lastfile("");
  set_name("");
  set_author("");
  set_homepage("");
  set_email("");
  set_version("");
}

// Sort the files received by the function alphabetically. They are passed by
// reference so you don't have to return the files. The files are parsed into
// a vector, sorted and then unparsed.
void sort_files (string &files)
{
  vector<string> names;
  string temp, str;
  int location;
    
  // Parse the files list
  location = files.find("/",0);
  while (location != (signed int)string::npos) 
  {
    names.push_back(files.substr(0,location));
    files.erase(0,location+1);
    location = files.find("/",0);
  }
    
  // Sort files
  vector<string>::iterator start = names.begin();
  vector<string>::iterator end = names.end();
  sort(start,end);
    
  // Un-parse files list
  for (unsigned int x=0; x<names.size(); x++)
    files += names.at(x) + "/";
}

// Add file to the sources, headers or other files. This function receives
// the short filename of the file that is to be added.
void ProjectSettings::add_file (string fn)
{
  // Add a source file
  if (fn.find(".c", 0) != string::npos)
  {
    sources += fn;
    sort_files(sources);
  }
  // Add a header file
  else if (fn.find(".h", 0) != string::npos)
  {
    headers += fn;
    sort_files(headers);
  }
  // Add an "other file"
  else
  {
    other_files += fn;
    sort_files(other_files);
  }
}

// Remove the file from the list. The function receives the file that is to be
// removed from the list.
void ProjectSettings::remove_file (string fn)
{
  // Delete the chosen source, header or other file's name
  if (fn.find(".c", 0) != string::npos)
  {
    int start = sources.find(fn, 0);
    sources.erase(start, fn.length()+1);
  }
  else if (fn.find(".h", 0) != string::npos)
  {
    int start = headers.find(fn, 0);
    headers.erase(start, fn.length()+1);
  }
  else
  {
    int start = other_files.find(fn, 0);
    other_files.erase(start, fn.length()+1);
  }
}

// Write the src/Makefile.am with the new sources. It reads in the file text,
// removes the files list & then adds the new sources list to the file.
void ProjectSettings::write_source_makefile()
{
  // Open the Makefile.am file & read in its text
  string fn = string(location) + "src/Makefile.am", temp, mfiles,
                text = read_file(fn);
        
  // Find & erase the current sources listing
  int loc1, loc2;
  loc1 = text.find("SOURCES = \\",0);
  loc2 = text.find("LDFLAGS",loc1);
  loc2 = text.rfind("\n",loc2);
  text.erase(loc1+11,loc2-loc1-11);
        
  // Add the source files to the string
  mfiles += "\n\t";
  temp = sources;
  while (!temp.empty())
  {
    loc2 = temp.find("/",0);
    mfiles += temp.substr(0,loc2) + "\\\n\t";
    temp.erase(0,loc2+1);
  }
        
  // Add the header files to the string
  temp = headers;
  while (!temp.empty())
  {
    loc2 = temp.find("/",0);
    mfiles += temp.substr(0,loc2) + "\\\n\t";
    temp.erase(0,loc2+1);
  }
        
  // Insert the files list into the Makefile.am text
  if (mfiles.length() > 5)
    mfiles.erase(mfiles.length()-3,3);
  
  mfiles += "\n";
  text.insert(loc1+11, mfiles);
        
  // Write the file back into itself
  write_file(fn, text); 
}

// Get the private properties of the project settings class
string ProjectSettings::get_name()
{ return name;
}

string ProjectSettings::get_author()
{ return author;
}

string ProjectSettings::get_email()
{ return email; 
}

string ProjectSettings::get_url()
{ return url; 
}

string ProjectSettings::get_version()
{ return version;
}

string ProjectSettings::get_source_files()
{ return sources; 
}

string ProjectSettings::get_header_files()
{ return headers; 
}

string ProjectSettings::get_other_files()
{ return other_files; 
}

string ProjectSettings::get_lastfile()
{ return lastfile; 
 }

string ProjectSettings::get_location()
{ return location; 
}

string ProjectSettings::get_template_name()
{ return template_name; 
}

// Set variables in the project class
void ProjectSettings::set_source_files (string files)
{ sources = files; 
}

void ProjectSettings::set_header_files (string files)
{ headers = files; 
}

void ProjectSettings::set_other_files (string files)
{ other_files = files; 
}

void ProjectSettings::set_lastfile (string file)
{ lastfile = file; 
}

void ProjectSettings::set_name (string str)
{ name = str; 
}

void ProjectSettings::set_author (string str)
{ author = str; 
}

void ProjectSettings::set_email (string str)
{ email = str; 
}

void ProjectSettings::set_homepage (string str)
{ url = str; 
}

void ProjectSettings::set_version (string str)
{ version = str; 
}
