/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "settings.h"
#include "utils.h"

bool get_attribute_bool(string, string&, bool);
string get_attribute_ustring(string, string&, string);
int get_attribute_int(string, string&, int);

// Get user environment settings from the openldev settings file. Load each of
// the properties, but also send a default if one of the attributes does not exist.
void EnvironmentSettings::load_settings_file()
{
  string txt, text, 
         config = string(g_get_home_dir()) + "/.openldev/openldev.settings";  

  // Open settings file
  text = read_file(config);

  // Get autosave & environment properties
  autosave = get_attribute_int("<autosave>", text, 10);
  width = get_attribute_int("<width>", text, 800);
  height = get_attribute_int("<height>", text, 600);
  treeview_width = get_attribute_int("<tvwidth>", text, 175);
  buildbox_height = get_attribute_int("<bbheight>", text, 175);
  undo = get_attribute_int("<undo>", text, 25);
  
  // Get lastproject and boolean properties
  lastproject = get_attribute_ustring("<lastproject>", text, "EMPTY");
  autoload = get_attribute_bool("<autoload>", text, true);
  replace_tabs = get_attribute_bool("<replacetabs>", text, true);
  line_numbers = get_attribute_bool("<lnumbers>", text, true);
  autoindent = get_attribute_bool("<autoindent>", text, true);
  wrap = get_attribute_bool("<wrap>", text, true);
  wrapsplit = get_attribute_bool("<wrapsplit>", text, false);
  wrapsplit = get_attribute_bool("<bracket>", text, true);
  rightmargin = get_attribute_bool("<rightmargin>", text, true);
  
  // Get compiler color settings
  error = get_attribute_ustring("<error>", text, "#FF0000");
  warning = get_attribute_ustring("<warning>", text, "#0000CC");
  
  // Get general attributes for openldev environment
  tabsize = get_attribute_int("<tabsize>", text, 2);  
  fontface = get_attribute_ustring("<fontface>", text, "Monospace");  
  fontsize = get_attribute_ustring("<fontsize>", text, "10");
  toolbar = get_attribute_ustring("<toolbar>", text, "NFOPSVRVSPCTCOPTSPUDRDSPFDGLSPCLBUEXSTSPCPPI");
}

// Write the settings file with the current environment settings.
void EnvironmentSettings::write_settings_file()
{
  string text, fn = g_get_home_dir();
  gchar tab[2], ht[5], wt[5], tv[5], bb[5], as[2], ud[3];
  
  // Convert integer properties to character arrays
  snprintf(tab, sizeof(tab), "%d", tabsize);
  snprintf(ht, sizeof(ht), "%d", height);
  snprintf(wt, sizeof(wt), "%d", width);
  snprintf(tv, sizeof(tv), "%d", treeview_width);
  snprintf(bb, sizeof(bb), "%d", buildbox_height);
  snprintf(as, sizeof(as), "%d", autosave);
  snprintf(ud, sizeof(ud), "%d", undo);
    
  // Create .ODP file text
  text = "<autosave>" + string(as) 
       + "*\n<error>" + error 
       + "*\n<warning>" + string(warning) 
       + "*\n<tabsize>" + tab 
       + "*\n<width>" + wt 
       + "*\n<height>" + ht 
       + "*\n<tvwidth>" + tv 
       + "*\n<toolbar>" + toolbar
       + "*\n<bbheight>" + bb
       + "*\n<undo>" + ud
       + "*\n<autoload>";
  
  if (autoload) 
    text += "T";
  else
    text += "F";
    
  if (replace_tabs) 
    text += "*\n<replacetabs>T";
  else 
    text += "*\n<replacetabs>F";
    
  if (line_numbers)
    text += "*\n<lnumbers>T";
  else
    text += "*\n<lnumbers>F";
  
  if (autoindent)
    text += "*\n<autoindent>T";
  else
    text += "*\n<autoindent>F";
  
  if (wrap)
    text += "*\n<wrap>T";
  else
    text += "*\n<wrap>F";
  
  if (wrapsplit)
    text += "*\n<wrapsplit>T";
  else
    text += "*\n<wrapsplit>F";
  
  if (brackets)
    text += "*\n<bracket>T";
  else
    text += "*\n<bracket>F";
  
  if (rightmargin)
    text += "*\n<rightmargin>T";
  else
    text += "*\n<rightmargin>F";
  
  text += "*\n<fontface>" + fontface 
       + "*\n<fontsize>" + fontsize 
       + "*\n<lastproject>" + lastproject + "*\n";
  
  fn += "/.openldev/openldev.settings";
  
  // Write the file
  write_file(fn, text);
}

// Create a new settings file by setting the default properties to the
// environment settings class and write the file.
void EnvironmentSettings::create_new_settings_file()
{
  set_autosave(10);
  set_error("#FF0000");
  set_warning("#0000CC");
  set_tabsize(2);
  set_undo(25);
  set_autoload(true);
  set_replace_tabs(true);
  set_line_numbers(true);
  set_auto_indent(true);
  set_wrap(true);
  set_wrapsplit(false);
  set_rightmargin(true);
  set_font_face("Monospace");
  set_font_size("10");
  set_toolbar("NFOPSVRVSPCTCOPTSPUDRDSPFDGLSPCLBUEXSTSPPFCPPI");
  set_lastproject("EMPTY");
  set_width(800);
  set_height(600);
  set_treeview_width(175);
  set_buildbox_height(175);
  write_settings_file();
}

// Get boolean data from the specified attribute in the text. This function
// receives the attribute to find, the text of the environment settings file
// and the value to return if the attribute does not exist.
bool get_attribute_bool (string attribute, 
                         string &text, 
                         bool fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    string txt = text.substr(start, len);
    if (txt == "T")
      return true;
    else
      return false;
  }
  else
    return fail;
}

// Get ustring data from the specified attribute in the text. This function
// receives the attribute to find, the text of the environment settings file
// and the value to return if the attribute does not exist.
string get_attribute_ustring (string attribute, 
                              string &text, 
                              string fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    return text.substr(start, len);
  }
  else
    return fail;
}

// Get integer data from the specified attribute in the text. This function
// receives the attribute to find, the text of the environment settings file
// and the value to return if the attribute does not exist.
int get_attribute_int (string attribute, 
                       string &text, 
                       int fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    string txt = text.substr(start, len);
    return atol(txt.c_str());
  }
  else
    return fail;
}
  
// Set private variables in the environment settings class
void EnvironmentSettings::set_autoload (bool load)
{ autoload = load; 
}

void EnvironmentSettings::set_rightmargin (bool margin)
{ rightmargin = margin; 
}

void EnvironmentSettings::set_auto_indent (bool indent)
{ autoindent = indent; 
}

void EnvironmentSettings::set_brackets (bool value)
{ brackets = value; 
}

void EnvironmentSettings::set_wrap (bool value)
{ wrap = value; 
}

void EnvironmentSettings::set_wrapsplit (bool value)
{ wrapsplit = value; 
}

void EnvironmentSettings::set_replace_tabs (bool var)
{ replace_tabs = var; 
}

void EnvironmentSettings::set_line_numbers (bool var)
{ line_numbers = var; 
}

void EnvironmentSettings::set_autosave (int save)
{ autosave = save; 
}

void EnvironmentSettings::set_tabsize (int size)
{ tabsize = size; 
}

void EnvironmentSettings::set_undo (int count)
{ undo = count; 
}

void EnvironmentSettings::set_width (int size)
{ width = size; 
}

void EnvironmentSettings::set_height (int size)
{ height = size; 
}

void EnvironmentSettings::set_treeview_width (int size)
{ treeview_width = size; 
}

void EnvironmentSettings::set_buildbox_height (int size)
{ buildbox_height = size; 
}

void EnvironmentSettings::set_font_size (string size)
{ fontsize = size; 
}

void EnvironmentSettings::set_error (string color)
{ error = color; 
}

void EnvironmentSettings::set_warning (string color)
{ warning = color; 
}

void EnvironmentSettings::set_font_face (string face)
{ fontface = face; 
}

void EnvironmentSettings::set_lastproject (string name)
{ lastproject = name; 
}

void EnvironmentSettings::set_toolbar (string tb)
{ toolbar = tb; 
}

// Get private variables in the environment settings class
int EnvironmentSettings::get_autosave()
{ return autosave; 
}

int EnvironmentSettings::get_tabsize()
{ return tabsize; 
}

int EnvironmentSettings::get_undo()
{ return undo; 
}

int EnvironmentSettings::get_width()
{ return width; 
}

int EnvironmentSettings::get_height()
{ return height;
}

int EnvironmentSettings::get_treeview_width()
{ return treeview_width; 
}

int EnvironmentSettings::get_buildbox_height()
{ return buildbox_height; 
}

bool EnvironmentSettings::get_autoload()
{ return autoload;
}

bool EnvironmentSettings::get_replace_tabs()
{ return replace_tabs; 
}

bool EnvironmentSettings::get_brackets()
{ return brackets; 
}

bool EnvironmentSettings::get_line_numbers()
{ return line_numbers; 
}

bool EnvironmentSettings::get_auto_indent()
{ return autoindent; 
}

bool EnvironmentSettings::get_wrap()
{ return wrap; 
}

bool EnvironmentSettings::get_wrapsplit()
{ return wrapsplit; 
}

bool EnvironmentSettings::get_rightmargin()
{ return rightmargin; 
}
 
string EnvironmentSettings::get_font_size()
{ return fontsize; 
}

string EnvironmentSettings::get_error_color()
{ return error; 
}

string EnvironmentSettings::get_warning_color()
{ return warning;
}

string EnvironmentSettings::get_font_face()
{ return fontface;
}

string EnvironmentSettings::get_lastproject()
{ return lastproject; 
}

string EnvironmentSettings::get_toolbar()
{ return toolbar; 
}
